; The Japanese Dragon Quest II supported hero names of up to 4 bytes; the English Dragon Warrior II expanded this to a maximum of 8 bytes, but with one exception,
; (printing the current hero's name in the full STATUS menu) the game's menu system was not updated to display 8-byte names.
;
; This patch updates the handlers for all of the menu control codes that print hero names to print 8 bytes instead of 4; since the one control code that originally
; printed 8 bytes is therefore redundant, it has been updated to print the current hero + 1's name instead. Note that since printing only the first 4 bytes of a
; hero's name is no longer supported, these changes require resizing almost every menu that displays a hero's name in order to avoid graphical glitches.
;
; Hero names are stored in two pieces:
; $0113 - $0117: the first four bytes of Midenhall's name + a terminating byte
; $0118 - $011C: the first four bytes of Cannock's name + a terminating byte
; $011D - $0121: the first four bytes of Moonbrooke's name + a terminating byte
; $0186 - $0189: the last four bytes of Midenhall's name (no terminating byte)
; $018A - $018D: the last four bytes of Cannock's name (no terminating byte)
; $018E - $0191: the last four bytes of Moonbrooke's name (no terminating byte)
;
; Menu control codes $B0-$B2 originally printed the first 4 bytes of Midenhall/Cannock/Moonbrooke's name; they now print all 8 bytes
; Menu control code $B3 originally printed the first 4 bytes of the current hero's name; it now prints all 8 bytes
; Menu control code $B4 originally printed all 8 bytes of the current hero's name; it now prints all 8 bytes of the current hero + 1's name
; Menu control codes $B5-$B7 originally printed the first 4 bytes of Midenhall's name from save game 1/2/3; they now print all 8 bytes
;
; From a player's perspective, Dragon Warrior II's menu system is far from complicated, but two minor improvements are adding the ability to display the selected
; hero's name in menus relating to that hero and backporting an upgrade from Dragon Warrior IV where the weapon shop's BUY menu displays an "E" next to the name of each
; hero who is able to equip the item you're trying to buy.
;
; This patch adds behaviour to the unused $9B - $9C menu control codes to print a hero's full 8 byte name with spaces ($5F or $60) replaced by top border tiles ($77):
; Menu control code $9B now prints all 8 bytes of the current hero's name with spaces replaced by borders
; Menu control code $9C now prints all 8 bytes of the current hero + 1's name with spaces replaced by borders
; (this is used when printing the caster's name in the SPELL menu - since Midenhall doesn't appear in the SPELL selection menu, Cannock/Moonbrooke's IDs are off by 1)
; Menu control codes $9D-$9F now print all 8 bytes of Midenhall/Cannock/Moonbrooke's name with spaces replaced by borders
;
; This patch adds behaviour to the unused $F0 - $F2 menu control codes to print an "E" if the Midenhall/Cannock/Moonbrooke can equip the item ID stored in $96;
; this assumes that $96 has already been set by the weapon shop's item selection menu's code.

norom

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; update menu control codes #$B0-#$B7
org $3EE34
base $EE24
warnpc $EE82

; menu control code #$B0-#$B7 handler: print selected hero's name, current hero + 1's name, or Midenhall's name from selected save slot
; indirect control flow target (via $F0D8)
cc_B0_7:
	LDA $609D		; menu function parameter
	CMP #$05
	BCS .cc_B567		; menu control code #$B5-#$B7 handler: print save game name
	CMP #$04
	LDA #$04		; LDA doesn't change C
	BCS .cc_B4		; menu control code #$B4 handler: print current hero + 1's name
; menu control code #$B0-#$B2 handler: print name of hero ID in $609D (Midenhall/Cannock/Moonbrooke)
; also menu control code #$B3 handler: print name of current hero (stored in $4A)
	JSR $F07A		; set maximum string length $60A0 to A; for control code #$B3, set menu function parameter $609D to the current hero's ID (for other control codes, $609D already contains the selected hero's ID); return $609D in A
.print_name
	ASL
	ASL
	PHA			; save A = $609D * 4 for later since the second half of each hero's name is packed in 4 bytes
	ADC $609D		; A = $609D * 5; the first half of each hero's name is packed in 5 bytes
	TAX			; read index
	LDY $60A0		; write index
; copy the first half of the hero's name in reverse to $0106 - $0103
-
	LDA $0113,X
	STA $0103,Y
	INX
	DEY
	BNE -
	PLA			; A = $609D * 4
	TAX			; read index
	LDY $60A0		; write index
; copy the second half of the hero's name in reverse to $0102 - $00FF
-
	LDA $0186,X
	STA $00FF,Y
	INX
	DEY
	BNE -
	LDA #$08
	STA $60A0		; maximum string length
	JMP $F0A5		; print string at $0100 in reverse to current menu position

.cc_B4
	STA $60A0		; maximum string length
	LDA $4A			; current hero ID
	ADC #$00		; we get here via BCS, so C is set
	STA $609D		; menu function parameter
	BNE .print_name

.cc_B567
	SBC #$05		; we got here via BCS, so we know C is set; 5/6/7 becomes 0/1/2
	PHA
	LDA #$08
	STA $60A0		; maximum string length
	PLA
	JSR $FE97		; read byte following JSR, parse it for bank and pointer index, execute ($8000,X) in selected bank, swap back in original bank
	db $CE			; $CE -> 0b11001110 -> 0b110, 0b01110 -> $06, $0E -> bank $06, $0E'th pointer -> pointer at $06:$801C, i.e. $06:$A88F: copy Midenhall's name from save slot in A to $0100 in reverse
	JMP $F0A5		; print string at $0100 in reverse to current menu position

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; update length of string to read for save game names
org $1A8A2

	LDY #$07


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; update menu control codes #$98-#$9F
org $3ED8A
base $ED7A
warnpc $EDBE

; menu control code #$98-#$9F handler: print party gold, crests, "ADVENTURE LOG", or selected/current hero's name with spaces replaced by top border tiles
; indirect control flow target (via $F0D2)
cc_98_F:
	LDA $609D	; menu function parameter
	BEQ cc_98
	CMP #$01	; #$99
	BNE cc_9A_F_jmp
; menu control code #$99 handler: print obtained crests
	LDA #$05	; 5 crests * 1 tile each
	STA $60A0	; maximum string length
	JSR $F19F	; initialize string buffer $0100-$010C with [space]s
	LDA $0112	; Crests found bit field (#$10 = Life, #$80 = Water, #$04 = Moon, #$02 = Stars, #$01 = Sun)
	AND #$1F	; only keep the 5 relevant bits
	LDX #$04	; maximum of 5 crests
	LDY #$59	; one less than Tile ID #$5A: [sun]
-
	INY		; crest tile ID
	LSR		; shift crest bit into C
	BCC +		; branch if crest not found
	PHA
	TYA
	STA $0100,X	; write crest tile to string buffer
	DEX
	PLA
+
	BNE -		; if more crests to copy, copy them
print_string:
	JMP $F0A5	; print string at $0100 in reverse to current menu position

; menu control code #$98 handler: print party gold
cc_98:
	JSR $F479	; print party gold to $0100 in decimal in reverse, replacing leading (now trailing) zeroes with spaces, leaving X at most significant digit
	JMP $F0A5	; print string at $0100 in reverse to current menu position

; menu control code #$9A handler: print "ADVENTURE LOG"
cc_9A:
	LDA #$0D	; length of "ADVENTURE LOG"
	STA $60A0	; maximum string length
	TAX
-
	LDA .LOG-1,X
	STA $00FF,X	; built-in offset from string copy buffer start at $0100
	DEX
	BNE -		; if more letters to copy, copy them
	BEQ print_string
.LOG
	db $2A,$32,$2F,$5F,$28,$35,$38,$37,$31,$28,$39,$27,$24	; "GOL ERUTNEVDA", i.e. "ADVENTURE LOG" backwards

cc_9A_F_jmp:
	JMP cc_9A_F	; jump to free space in bank 1; relies on bank 1 already being loaded, which is the case while processing menu strings


org $7F73
base $BF63
warnpc $BFCF

cc_9A_F:
	CMP #$02	; #$9A
	BEQ .cc_9A_jmp
	CMP #$04	; #$9C
	BEQ .cc_9C
; else: #$9B (current hero) or #$9D-#$9F (specific hero)
	LDA #$04	; hero name pieces are 4 bytes each
	JSR $F07A	; for #$9B, set $609D = $4A (menu selection index, which happens to be the same as hero ID)
.print_name
	ASL
	ASL
	PHA		; save A = $609D * 4 for later use
	ADC $609D	; A = $609D * 5
	TAX
	LDY $60A0	; maximum string length
; copy the first half of the hero's name in reverse to $0106 - $0103, replacing spaces (#$5F/#$60) with top borders (#$77)
-
	LDA $0113,X	; first 4 letters of name
	CMP #$5F	; Cannock/Moonbrooke's names use #$5F for their trailing spaces
	BCC +
	CMP #$61	; Midenhall's name uses #$60 for their trailing spaces
	BCS +
	LDA #$77	; replace #$5F or #$60 with #$77 (top border)
+
	STA $0103,Y	; built-in offset from string copy buffer start at $0100
	INX
	DEY
	BNE -		; if more letters to copy, copy them
	PLA		; A = $609D * 4
	TAX
	LDY $60A0	; maximum string length
; copy the second half of the hero's name in reverse to $0102 - $00FF, replacing spaces (#$5F/#$60) with top borders (#$77)
-
	LDA $0186,X	; last 4 letters of name
	CMP #$5F	; Cannock/Moonbrooke's names use #$5F for their trailing spaces
	BCC +
	CMP #$61	; Midenhall's name uses #$60 for their trailing spaces
	BCS +
	LDA #$77	; replace #$5F or #$60 with #$77 (top border)
+
	STA $00FF,Y	; built-in offset from string copy buffer start at $0100
	INX
	DEY
	BNE -		; if more letters to copy, copy them
	LDA #$08	; length of full name
	STA $60A0	; maximum string length
	JMP $F0A5	; print string at $0100 in reverse to current menu position

.cc_9C
; the field SPELL menu does not list Midenhall (since he can't cast spells), so +1 to get the right hero IDs for Cannock/Moonbrooke
	LDA #$04	; hero name pieces are 4 bytes each
	STA $60A0	; maximum string length
	LDA $4A		; menu selection index; for #$9C this is Cannock = 0, Moonbrooke = 1
	ADC #$00	; we get here via BEQ from a CMP, so we know C is already set
	STA $609D	; for #$9C, menu selection index + 1 = hero ID
	BNE .print_name

.cc_9A_jmp:
	JMP cc_9A	; jump back to bank $0F

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; handler for menu control codes #$F0 - #$F8
menu_F0_F8_handler:
; F0-F2 are only used as part of menu IDs #$4A/#$4B
	LDA $609D	; menu function parameter
	STA $97
	LDA #$06
	JSR check_equippability	; given item ID in $96 and hero ID in $97, set A to #$80 if hero can equip item, #$00 otherwise
	BPL .done
	LDA #$28	; Tile ID #$28: E
	STA $609B	; menu text
	JMP $EFE4	; print one byte of menu text
.done:
	RTS
;+21 bytes



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; repurpose unused menu control codes F0-F2 to print "E" if corresponding hero is able to equip selected item
; update pointer to handler for menu control codes #$F0 - #$F8
; bank $0F
org $3F0F8
base $F0E8

	dw menu_F0_F8_handler

; switch call to display weapon shop BUY hero select menu to new code
; bank $06
org $1831E
base $830E

	JSR display_new_menu

; new code to display appropriate weapon shop BUY hero select menu, separate from the old menu that was shared between weapon shop BUY, item shop BUY, item shop SELL, and UNCURSE, DETOXICATE, and REVIVAL
; bank $06
org $1BFC7
base $BFB7

display_new_menu:
	JSR $F65A	; set A and X to the current number of party members
	BEQ .done	; if just Midenhall, no need to display hero select menu
	ORA #$4A	; set A to #$4B or #$4A
	JMP $EB89	; open menu specified by A
.done:
	RTS
;+11 bytes

; swap in bank 6 to call code there, then swap back to bank 1
; bank $0F
org $3FFEF
base $FFDF

check_equippability:
	JSR $FFBB
	JSR $A3A3	; given item ID in $96 and hero ID in $97, set A to #$80 if hero can equip item, #$00 otherwise
	PHA
	LDA #$01
	JSR $FFBB
	PLA
	RTS
;+14 bytes